@@ -1,15 +1,12 @@
 from django.conf import settings
 from django.contrib.auth.models import User, Group
-from django.contrib.contenttypes.models import ContentType
-from django.db import IntegrityError
 from django.test import TestCase
 from django.test.client import Client
 
 from object_permissions import *
 from object_permissions.registration import TestModel, TestModelChild, \
     TestModelChildChild, UnknownPermissionException
-from object_permissions.signals import view_add_user, view_remove_user, \
-    view_edit_user, view_group_edited, view_group_created, view_group_deleted
+from object_permissions.signals import view_edit_user
 
 from object_permissions.templatetags.object_permission_tags import \
                 number_group_admins
@@ -17,10 +14,17 @@ from object_permissions.templatetags.object_permission_tags import \
 __all__ = ('TestGroups','TestGroupViews')
 
 
+# XXX set global vars to keep tests cleaner
+perms = set(['Perm1', 'Perm2', 'Perm3', 'Perm4'])
+user0 = None
+user1 = None
+object0 = None
+object1 = None
+
 class TestGroups(TestCase):
-    perms = set(['Perm1', 'Perm2', 'Perm3', 'Perm4'])
-    
+
     def setUp(self):
+        global user0, user1, object0, object1
         self.tearDown()
         
         User(id=1, username='anonymous').save()
@@ -37,24 +41,24 @@ class TestGroups(TestCase):
         object0.save()
         object1 = TestModel.objects.create(name='test1')
         object1.save()
-        
-        dict_ = globals()
-        dict_['user0']=user0
-        dict_['user1']=user1
-        dict_['object0']=object0
-        dict_['object1']=object1
-        dict_['perms']=self.perms
     
     def tearDown(self):
+        global user0, user1, object0, object1
+
         User.objects.all().delete()
         TestModel.objects.all().delete()
         TestModelChild.objects.all().delete()
         TestModelChildChild.objects.all().delete()
         Group.objects.all().delete()
 
+        user0 = None
+        user1 = None
+        object0 = None
+        object1 = None
+
     def test_trivial(self):
         """ Test instantiating a Group """
-        group = Group()
+        Group()
 
     def test_save(self, name='test', user=None):
         """ Test saving an Group """
@@ -247,11 +251,10 @@ class TestGroups(TestCase):
         """
         group0 = self.test_save('TestGroup0')
         group1 = self.test_save('TestGroup1')
-        perms1 = self.perms
+        perms1 = perms
         perms2 = set(['Perm1', 'Perm2'])
         perms3 = set(['Perm2', 'Perm3'])
         perms4 = []
-        
         # grant single property
         set_group_perms(group0, perms1, object0)
         self.assertEqual(perms1, set(get_group_perms(group0, object0)))
@@ -284,7 +287,54 @@ class TestGroups(TestCase):
         self.assertEqual(perms4, get_group_perms(group0, object0))
         self.assertEqual(perms2, set(get_group_perms(group0, object1)))
         self.assertEqual(perms1, set(get_group_perms(group1, object0)))
-    
+
+    def test_get_perms(self):
+        """
+        tests retrieving list of perms
+
+        Verifies:
+            * No Perms returns empty list
+            * some perms returns just that list
+            * all perms returns all perms
+        """
+        group0 = self.test_save('TestGroup0', user0)
+        group1 = self.test_save('TestGroup1', user0)
+
+        self.assertEqual([], group0.get_perms(object0))
+
+        group0.grant('Perm1', object0)
+        group1.grant('Perm3', object1)
+        group1.grant('Perm4', object1)
+
+        self.assertEqual(['Perm1'], group0.get_perms(object0))
+
+        perms = group0.get_perms(object1)
+        self.assertEqual(2, len(perms))
+        self.assertEqual(set(['Perm3','Perm4']), set(perms))
+
+    def test_group_get_perms_any(self):
+        """
+        tests retrieving list of perms across any instance of a model
+
+        Verifies:
+            * No Perms returns empty list
+            * some perms returns just that list
+            * all perms returns all perms
+        """
+        group0 = self.test_save('TestGroup0')
+        group1 = self.test_save('TestGroup1')
+
+        self.assertEqual([], group0.get_perms_any(TestModel))
+
+        group0.grant('Perm1', object0)
+        group0.grant('Perm3', object1)
+        group0.grant('Perm4', object1)
+        group1.grant('Perm2', object0)
+
+        perms = group0.get_perms_any(TestModel)
+        self.assertEqual(3, len(perms))
+        self.assertEqual(set(['Perm1', 'Perm3', 'Perm4']), set(perms))
+
     def test_has_perm(self):
         """
         Additional tests for has_perms
@@ -423,6 +473,59 @@ class TestGroups(TestCase):
         self.assert_(group0 in get_groups_all(object0, ['Perm1','Perm2']))
         self.assert_(group0 in get_groups_all(object1, ['Perm1','Perm3']))
 
+    def test_user_get_perms(self):
+        """
+        tests retrieving list of perms
+
+        Verifies:
+            * No Perms returns empty list
+            * some perms returns just that list
+            * all perms returns all perms
+        """
+        group0 = self.test_save('TestGroup0', user0)
+        group1 = self.test_save('TestGroup1', user0)
+
+        self.assertEqual([], user0.get_perms(object0))
+
+        group0.grant('Perm1', object0)
+        group1.grant('Perm3', object1)
+        group1.grant('Perm4', object1)
+
+        self.assertEqual(['Perm1'], user0.get_perms(object0))
+
+        perms = user0.get_perms(object1)
+        self.assertEqual(2, len(perms))
+        self.assertEqual(set(['Perm3','Perm4']), set(perms))
+
+        # test excluding groups
+        self.assertEqual([], user0.get_perms(object0, False))
+
+
+    def test_user_get_perms_any(self):
+        """
+        tests retrieving list of perms across any instance of a model
+
+        Verifies:
+            * No Perms returns empty list
+            * some perms returns just that list
+            * all perms returns all perms
+        """
+        group0 = self.test_save('TestGroup0', user0)
+        group1 = self.test_save('TestGroup1', user0)
+
+        self.assertEqual([], user0.get_perms_any(TestModel))
+
+        group0.grant('Perm1', object0)
+        group1.grant('Perm3', object1)
+        group1.grant('Perm4', object1)
+
+        perms = user0.get_perms_any(TestModel)
+        self.assertEqual(3, len(perms))
+        self.assertEqual(set(['Perm1', 'Perm3', 'Perm4']), set(perms))
+
+        # exclude group perms
+        self.assertEqual([], user0.get_perms_any(TestModel, False))
+
     def test_user_get_objects_any_perms(self):
         """
         Test filtering objects
@@ -691,7 +794,27 @@ class TestGroups(TestCase):
         query = user0.get_objects_all_perms(TestModelChild, perms=['Perm1'], groups=False, parent=['Perm1'])
         self.assert_(child0 in query)
         self.assertEqual(1, len(query))
-    
+    def test_get_perms(self):
+        """
+        tests retrieving list of perms across any instance of a model
+
+        Verifies:
+            * No Perms returns empty list
+            * some perms returns just that list
+            * all perms returns all perms
+        """
+        self.assertEqual([], user0.get_perms(object0))
+
+        grant(user0, 'Perm1', object0)
+        grant(user0, 'Perm3', object1)
+        grant(user0, 'Perm4', object1)
+        grant(user1, 'Perm2', object0)
+
+        self.assertEqual(['Perm1'], user0.get_perms(object0))
+
+        perms = user0.get_perms(object1)
+        self.assertEqual(2, len(perms))
+        self.assertEqual(set(['Perm3','Perm4']), set(perms))
     def test_group_has_any_perms_on_model(self):
         """
         Test checking if a user has any of the perms on any instance of the model
@@ -1007,508 +1130,42 @@ class TestGroups(TestCase):
 
 
 class TestGroupViews(TestCase):
-    perms = ['Perm1', 'Perm2', 'Perm3', 'Perm4']
-    
+
     def setUp(self):
+        global user0, user1, object0, object1
         self.tearDown()
         
         User(id=1, username='anonymous').save()
         settings.ANONYMOUS_USER_ID=1
         
-        self.user = User(id=2, username='tester0')
-        self.user.set_password('secret')
-        self.user.save()
-        self.user1 = User(id=3, username='tester1')
-        self.user1.set_password('secret')
-        self.user1.save()
-        
-        self.object0 = TestModel.objects.create(name='test0')
-        self.object0.save()
-        self.object1 = TestModel.objects.create(name='test1')
-        self.object1.save()
+        user0 = User(id=2, username='tester0')
+        user0.set_password('secret')
+        user0.save()
+        user1 = User(id=3, username='tester1')
+        user1.set_password('secret')
+        user1.save()
+
+        object0 = TestModel.objects.create(id=10, name='test0')
+        object0.save()
+        object1 = TestModel.objects.create(id=13, name='test1')
+        object1.save()
     
     def tearDown(self):
+        global user0, user1, object0, object1
         User.objects.all().delete()
         TestModel.objects.all().delete()
         Group.objects.all().delete()
 
+        user0 = None
+        user1 = None
+        object0 = None
+        object1 = None
+
     def test_save(self, name='test'):
         """ Test saving an Group """
         group = Group(name=name)
         group.save()
         return group
-
-    def test_number_of_group_admins(self):
-        """
-        Test template tag used in list of Groups view
-        """
-
-        group0 = self.test_save(name='group1')
-        group1 = self.test_save(name='group2')
-        user0 = self.user
-        user1 = self.user1
-        
-        group0.user_set.add(user0)
-        group0.user_set.add(user1)
-        user0.grant("admin", group0)
-        group1.user_set.add(user0)
-        group1.user_set.add(user1)
-
-        self.assertEqual(number_group_admins(group0), 1)
-        self.assertEqual(number_group_admins(group1), 0)
-        user1.grant("admin", group1)
-        self.assertEqual(number_group_admins(group1), 1)
-        user1.grant("admin", group0)
-        self.assertEqual(number_group_admins(group0), 2)
-    
-    def test_view_list(self):
-        """
-        Test viewing list of Groups
-        """
-        user = self.user
-        group = self.test_save()
-        group0 = self.test_save(name='group1')
-        group1 = self.test_save(name='group2')
-        group2 = self.test_save(name='group3')
-        c = Client()
-        url = '/groups/'
-        
-        # anonymous user
-        response = c.get(url, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertTemplateUsed(response, 'registration/login.html')
-        
-        # unauthorized user (user with admin on no groups)
-        self.assert_(c.login(username=user.username, password='secret'))
-        response = c.get(url)
-        self.assertEqual(403, response.status_code)
-        
-        # authorized (permission)
-        user.grant('admin', group)
-        user.grant('admin', group1)
-        response = c.get(url)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/list.html')
-        groups = response.context['groups']
-        self.assert_(group in groups)
-        self.assert_(group1 in groups)
-        self.assertEqual(2, len(groups))
-        
-        # authorized (superuser)
-        user.revoke('admin', group0)
-        user.revoke('admin', group1)
-        user.is_superuser = True
-        user.save()
-        response = c.get(url)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/list.html')
-        groups = response.context['groups']
-        self.assert_(group in groups)
-        self.assert_(group0 in groups)
-        self.assert_(group1 in groups)
-        self.assert_(group2 in groups)
-        self.assertEqual(4, len(groups))
-    
-    def test_view_detail(self):
-        """
-        Test Viewing the detail for a Group
-        
-        Verifies:
-            * 200 returned for valid group
-            * 404 returned for invalid group
-        """
-        user = self.user
-        group = self.test_save()
-        c = Client()
-        url = '/group/%s/'
-        args = group.id
-        
-        # anonymous user
-        response = c.get(url % args, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertTemplateUsed(response, 'registration/login.html')
-        
-        # unauthorized user
-        self.assert_(c.login(username=user.username, password='secret'))
-        response = c.get(url % args)
-        self.assertEqual(403, response.status_code)
-        
-        # invalid usergroup
-        response = c.get(url % "DoesNotExist")
-        self.assertEqual(404, response.status_code)
-        
-        # authorized (permission)
-        grant(user, 'admin', group)
-        response = c.get(url % args)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/detail.html')
-        
-        # authorized (superuser)
-        response = c.get(url % args)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/detail.html')
-
-    def test_view_edit(self):
-        user = self.user
-        group = self.test_save()
-        c = Client()
-        url = '/group/%s/'
-        
-        # anonymous user
-        response = c.post(url % group.id, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertTemplateUsed(response, 'registration/login.html')
-        
-        # unauthorized user
-        self.assert_(c.login(username=user.username, password='secret'))
-        response = c.post(url % group.id)
-        self.assertEqual(403, response.status_code)
-        
-        # invalid usergroup
-        response = c.post(url % "DoesNotExist")
-        self.assertEqual(404, response.status_code)
-        
-        # get form - authorized (permission)
-        # XXX need to implement Class wide permission for creating editing groups
-        #grant(user, 'admin', group)
-        #response = c.post(url % group.id)
-        #self.assertEqual(200, response.status_code)
-        #self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        #self.assertTemplateUsed(response, 'group/edit.html')
-        
-        # get form - authorized (permission)
-        grant(user, 'admin', group)
-        response = c.post(url % group.id)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/edit.html')
-        
-        # get form - authorized (superuser)
-        user.revoke('admin', group)
-        user.is_superuser = True
-        user.save()
-        response = c.post(url % group.id)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/edit.html')
-        
-        # missing name
-        data = {'id':group.id}
-        response = c.post(url % group.id, data)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        
-        # setup signal
-        self.signal_editor = self.signal_group = None
-        def callback(sender, editor, **kwargs):
-            self.signal_user = user
-            self.signal_group = sender
-        view_group_edited.connect(callback)
-        
-        # successful edit
-        data = {'id':group.id, 'name':'EDITED_NAME'}
-        response = c.post(url % group.id, data, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/group_row.html')
-        group = Group.objects.get(id=group.id)
-        self.assertEqual('EDITED_NAME', group.name)
-        
-        # check signal set properties
-        self.assertEqual(group, self.signal_group)
-        self.assertEqual(user, self.signal_user)
-    
-    def test_view_create(self):
-        """
-        Test creating a new Group
-        """
-        user = self.user
-        group = self.test_save()
-        c = Client()
-        url = '/group/'
-        
-        # anonymous user
-        response = c.post(url, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertTemplateUsed(response, 'registration/login.html')
-        
-        # unauthorized user
-        self.assert_(c.login(username=user.username, password='secret'))
-        response = c.post(url)
-        self.assertEqual(403, response.status_code)
-        
-        # get form - authorized (permission)
-        # XXX need to implement Class level permissions
-        #grant(user, 'admin', group)
-        #response = c.post(url % group.id)
-        #self.assertEqual(200, response.status_code)
-        #self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        #self.assertTemplateUsed(response, 'group/edit.html')
-        
-        # get form - authorized (superuser)
-        user.revoke('admin', group)
-        user.is_superuser = True
-        user.save()
-        response = c.post(url)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/edit.html')
-        
-        # missing name
-        data = {'id':group.id}
-        response = c.post(url)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/edit.html')
-        
-        # setup signal
-        self.signal_editor = self.signal_group = None
-        def callback(sender, editor, **kwargs):
-            self.signal_user = user
-            self.signal_group = sender
-        view_group_created.connect(callback)
-        
-        # successful edit
-        data = {'name':'ADD_NEW_GROUP'}
-        response = c.post(url, data, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/group_row.html')
-        self.assert_(Group.objects.filter(name='ADD_NEW_GROUP').exists())
-        
-        # check signal set properties
-        self.assertEqual(Group.objects.get(name='ADD_NEW_GROUP'), self.signal_group)
-        self.assertEqual(user, self.signal_user)
-    
-    def test_view_delete(self):
-        """
-        Test deleting a usergroup
-        
-        Verifies:
-            * group is deleted
-            * all associated permissions are deleted
-        """
-        user = self.user
-        group0 = self.test_save()
-        group1 = self.test_save(name='test2')
-        c = Client()
-        url = '/group/%s/'
-        
-        # anonymous user
-        response = c.delete(url % group0.id, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertTemplateUsed(response, 'registration/login.html')
-        
-        # unauthorized user
-        self.assert_(c.login(username=user.username, password='secret'))
-        response = c.delete(url % group0.id)
-        self.assertEqual(403, response.status_code)
-        
-        # invalid usergroup
-        response = c.delete(url % "DoesNotExist")
-        self.assertEqual(404, response.status_code)
-        
-        # get form - authorized (permission)
-        grant(user, 'admin', group0)
-        response = c.delete(url % group0.id)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        self.assertFalse(Group.objects.filter(id=group0.id).exists())
-        self.assertEqual('1', response.content)
-        
-        # setup signal
-        self.signal_editor = self.signal_group = None
-        def callback(sender, editor, **kwargs):
-            self.signal_user = user
-            self.signal_group = sender
-        view_group_deleted.connect(callback)
-        
-        # get form - authorized (superuser)
-        user.is_superuser = True
-        user.save()
-        response = c.delete(url % group1.id)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        self.assertFalse(Group.objects.filter(id=group1.id).exists())
-        self.assertEqual('1', response.content)
-        
-        # check signal set properties
-        self.assertEqual(group1.name, self.signal_group.name)
-        self.assertEqual(user, self.signal_user)
-    
-    def test_view_add_user(self):
-        """
-        Test view for adding a user:
-        
-        Verifies:
-            * request from unauthorized user results in 403
-            * GET returns a 200 code, response is html
-            * POST with a user id adds user, response is html for user
-            * POST without user id returns error as json
-            * POST for invalid user id returns error as json
-            * adding user a second time returns error as json
-        """
-        user = self.user
-        group = self.test_save()
-        c = Client()
-        url = '/group/%d/user/add/'
-        args = group.id
-        
-        # anonymous user
-        response = c.get(url % args, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertTemplateUsed(response, 'registration/login.html')
-        
-        # unauthorized
-        self.assert_(c.login(username=user.username, password='secret'))
-        response = c.get(url % args)
-        self.assertEqual(403, response.status_code)
-        response = c.post(url % args)
-        self.assertEqual(403, response.status_code)
-        
-        # authorized get (perm granted)
-        grant(user, 'admin', group)
-        response = c.get(url % args)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/add_user.html')
-        
-        # authorized get (superuser)
-        revoke(user, 'admin', group)
-        user.is_superuser = True
-        user.save()
-        response = c.get(url % args)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/group/add_user.html')
-        
-        # missing user id
-        response = c.post(url % args)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        
-        # invalid user
-        response = c.post(url % args, {'user':0})
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        
-        # setup signal
-        self.signal_sender = self.signal_user = self.signal_obj = None
-        def callback(sender, user, obj, **kwargs):
-            self.signal_sender = sender
-            self.signal_user = user
-            self.signal_obj = obj
-        view_add_user.connect(callback)
-        
-        # valid post
-        data = {'user':user.id}
-        response = c.post(url % args, data)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/permissions/user_row.html')
-        self.assert_(group.user_set.filter(id=user.id).exists())
-        
-        # check signal fired
-        self.assertEqual(self.signal_sender, user)
-        self.assertEqual(self.signal_user, user)
-        self.assertEqual(self.signal_obj, group)
-        view_add_user.disconnect(callback)
-        
-        # same user again
-        response = c.post(url % args, data)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        self.assertEquals(group.user_set.filter(id=user.id).count(), 1)
-    
-    def test_view_remove_user(self):
-        """
-        Test view for adding a user:
-        
-        Verifies:
-            * GET redirects user to 405
-            * POST with a user id remove user, returns 1
-            * POST without user id returns error as json
-            * users lacking perms receive 403
-            * removing user not in group returns error as json
-            * removing user that does not exist returns error as json
-            * user loses all permissions when removed from group
-        """
-        user = self.user
-        group = self.test_save()
-        c = Client()
-        group.user_set.add(user)
-        url = '/group/%d/user/remove/'
-        args = group.id
-        
-        # anonymous user
-        response = c.get(url % args, follow=True)
-        self.assertEqual(200, response.status_code)
-        self.assertTemplateUsed(response, 'registration/login.html')
-        
-        # invalid permissions
-        self.assert_(c.login(username=user.username, password='secret'))
-        response = c.get(url % args)
-        self.assertEqual(403, response.status_code)
-        response = c.post(url % args)
-        self.assertEqual(403, response.status_code)
-        
-        # authorize and login
-        grant(user, 'admin', group)
-        
-        # invalid method
-        response = c.get(url % args)
-        self.assertEqual(405, response.status_code)
-        
-        # valid request (perm)
-        data = {'user':user.id}
-        response = c.post(url % args, data)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        self.assertEqual('1', response.content)
-        self.assertFalse(group.user_set.filter(id=user.id).exists())
-        self.assertEqual([], user.get_perms(group))
-        
-        # setup signal
-        self.signal_sender = self.signal_user = self.signal_obj = None
-        def callback(sender, user, obj, **kwargs):
-            self.signal_sender = sender
-            self.signal_user = user
-            self.signal_obj = obj
-        view_remove_user.connect(callback)
-        
-        # valid request (superuser)
-        revoke(user, 'admin', group)
-        user.is_superuser = True
-        user.save()
-        group.user_set.add(user)
-        response = c.post(url % args, data)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        self.assertEqual('1', response.content)
-        self.assertFalse(group.user_set.filter(id=user.id).exists())
-        
-        # check signal fired
-        self.assertEqual(self.signal_sender, user)
-        self.assertEqual(self.signal_user, user)
-        self.assertEqual(self.signal_obj, group)
-        view_remove_user.disconnect(callback)
-        
-        # remove user again
-        response = c.post(url % args, data)
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        self.assertFalse(group.user_set.filter(id=user.id).exists())
-        self.assertNotEqual('1', response.content)
-        
-        # remove invalid user
-        response = c.post(url % args, {'user':0})
-        self.assertEqual(200, response.status_code)
-        self.assertEquals('application/json', response['content-type'])
-        self.assertNotEqual('1', response.content)
     
     def test_view_update_permissions(self):
         """
@@ -1527,15 +1184,14 @@ class TestGroupViews(TestCase):
             * POST with a group_id adds user, response is html for user
             * perms added to appropriate models
         """
-        user = self.user
         group = self.test_save()
-        group.user_set.add(user)
+        group.user_set.add(user0)
         group1 = self.test_save('other_group')
         
         c = Client()
         url = '/group/%d/permissions/user/%s/'
         url_post = '/group/%d/permissions/'
-        args = (group.id, user.id)
+        args = (group.id, user0.id)
         args_post = group.id
         
         # anonymous user
@@ -1544,24 +1200,24 @@ class TestGroupViews(TestCase):
         self.assertTemplateUsed(response, 'registration/login.html')
         
         # unauthorized
-        self.assert_(c.login(username=user.username, password='secret'))
+        self.assert_(c.login(username=user0.username, password='secret'))
         response = c.get(url % args)
         self.assertEqual(403, response.status_code)
         response = c.post(url % args)
         self.assertEqual(403, response.status_code)
         
         # authorized post (perm granted)
-        grant(user, 'admin', group)
-        response = c.get(url % args, {'user':user.id, 'obj':self.object0.pk})
+        grant(user0, 'admin', group)
+        response = c.get(url % args, {'user':user0.id, 'obj':group.pk})
         self.assertEqual(200, response.status_code)
         self.assertEquals('text/html; charset=utf-8', response['content-type'])
         self.assertTemplateUsed(response, 'object_permissions/permissions/form.html')
         
         # authorized post (superuser)
-        revoke(user, 'admin', group)
-        user.is_superuser = True
-        user.save()
-        response = c.get(url % args, {'user':user.id})
+        revoke(user0, 'admin', group)
+        user0.is_superuser = True
+        user0.save()
+        response = c.get(url % args, {'user':user0.id})
         self.assertEqual(200, response.status_code)
         self.assertEquals('text/html; charset=utf-8', response['content-type'])
         self.assertTemplateUsed(response, 'object_permissions/permissions/form.html')
@@ -1571,32 +1227,32 @@ class TestGroupViews(TestCase):
         self.assertEqual(404, response.status_code)
         
         # invalid group (GET)
-        response = c.get(url % (-1, user.id))
+        response = c.get(url % (-1, user0.id))
         self.assertEqual(404, response.status_code)
         
         # invalid user (POST)
-        data = {'permissions':['admin'], 'user':-1, 'obj':self.object0.pk}
+        data = {'permissions':['admin'], 'user':-1, 'obj':group.pk}
         response = c.post(url_post % args_post, data)
         self.assertEqual(200, response.status_code)
         self.assertEquals('application/json', response['content-type'])
         self.assertNotEquals('1', response.content)
         
         # invalid group (POST)
-        data = {'permissions':['admin'], 'group':-1, 'obj':self.object0.pk}
+        data = {'permissions':['admin'], 'group':-1, 'obj':group.pk}
         response = c.post(url_post % args_post, data)
         self.assertEqual(200, response.status_code)
         self.assertEquals('application/json', response['content-type'])
         self.assertNotEquals('1', response.content)
         
         # user and group (POST)
-        data = {'permissions':['admin'], 'user':user.id, 'group':group1.id, 'obj':self.object0.pk}
+        data = {'permissions':['admin'], 'user':user0.id, 'group':group1.id, 'obj':group.pk}
         response = c.post(url_post % args_post, data)
         self.assertEqual(200, response.status_code)
         self.assertEquals('application/json', response['content-type'])
         self.assertNotEquals('1', response.content)
         
         # invalid permission
-        data = {'permissions':['DoesNotExist'], 'user':user.id, 'obj':self.object0.pk}
+        data = {'permissions':['DoesNotExist'], 'user':user0.id, 'obj':group.pk}
         response = c.post(url_post % args_post, data)
         self.assertEqual(200, response.status_code)
         self.assertEquals('application/json', response['content-type'])
@@ -1611,45 +1267,33 @@ class TestGroupViews(TestCase):
         view_edit_user.connect(callback)
         
         # valid post user
-        data = {'permissions':['admin'], 'user':user.id, 'obj':self.object0.pk}
+
+        data = {'permissions':['admin'], 'user':user0.id, 'obj':group.pk}
         response = c.post(url_post % args_post, data)
+
         self.assertEqual(200, response.status_code)
         self.assertEquals('text/html; charset=utf-8', response['content-type'])
         self.assertTemplateUsed(response, 'object_permissions/permissions/user_row.html')
-        self.assert_(user.has_perm('admin', group))
-        self.assertEqual(['admin'], get_user_perms(user, group))
+        self.assert_(user0.has_perm('admin', group))
+        self.assertEqual(['admin'], get_user_perms(user0, group))
         
         # check signal fired with correct values
-        self.assertEqual(self.signal_sender, user)
-        self.assertEqual(self.signal_user, user)
+        self.assertEqual(self.signal_sender, user0)
+        self.assertEqual(self.signal_user, user0)
         self.assertEqual(self.signal_obj, group)
         view_edit_user.disconnect(callback)
         
         # valid post no permissions user
-        data = {'permissions':[], 'user':user.id, 'obj':self.object0.pk}
+        data = {'user':user0.id, 'obj':group.pk}
         response = c.post(url_post % args_post, data)
-        self.assertEqual(200, response.status_code)
-        self.assertEqual([], get_user_perms(user, group))
-        
-        # valid post group
-        data = {'permissions':['admin'], 'group':group1.id, 'obj':self.object0.pk}
-        response = c.post(url_post % args_post, data)
-        self.assertEqual(200, response.status_code)
         self.assertEquals('text/html; charset=utf-8', response['content-type'])
-        self.assertTemplateUsed(response, 'object_permissions/permissions/user_row.html')
-        self.assertEqual(['admin'], group1.get_perms(group))
-        
-        # valid post no permissions group
-        data = {'permissions':[], 'group':group1.id, 'obj':self.object0.pk}
-        response = c.post(url_post % args_post, data)
         self.assertEqual(200, response.status_code)
-        self.assertEqual([], group1.get_perms(group))
-    
+        self.assertEqual([], get_user_perms(user0, group))
+
     def test_permissions_all(self):
         """ tests groups.permissions_all() """
-        user = self.user
         group = self.test_save()
-        group.user_set.add(user)
+        group.user_set.add(user0)
         group1 = self.test_save('other_group')
         
         url = '/group/%s/permissions/all'
@@ -1661,7 +1305,7 @@ class TestGroupViews(TestCase):
         self.assertTemplateUsed(response, 'registration/login.html')
         
         # unauthorized user - wrong group
-        self.assert_(c.login(username=user.username, password='secret'))
+        self.assert_(c.login(username=user0.username, password='secret'))
         response = c.get(url % group1.pk)
         self.assertEqual(403, response.status_code)
         
@@ -1670,14 +1314,14 @@ class TestGroupViews(TestCase):
         self.assertEqual(404, response.status_code)
         
         # authorized user - group member
-        self.assert_(c.login(username=user.username, password='secret'))
+        self.assert_(c.login(username=user0.username, password='secret'))
         response = c.get(url % group.pk)
         self.assertEqual(200, response.status_code)
         self.assertTemplateUsed(response, 'object_permissions/permissions/objects.html')
         
         # superuser
-        user.is_superuser = True
-        user.save()
+        user0.is_superuser = True
+        user0.save()
         response = c.get(url % group1.pk)
         self.assertEqual(200, response.status_code)
         self.assertTemplateUsed(response, 'object_permissions/permissions/objects.html')
